#!/bin/ksh
#
#	$Id: spamdba.ksh,v 1.31 2015/02/02 19:13:11 craski-shell_86 Exp $
#
# vim: tabstop=4 shiftwidth=4 softtabstop=4 noexpandtab
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
# Copyright (c) 2006, 2013-2015 Craig R. Skinner <skinner@britvault.co.uk>
#
# Permission to use, copy, modify, and distribute this software for any
# purpose with or without fee is hereby granted, provided that the above
# copyright notice and this permission notice appear in all copies.
#
# THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
# WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
# ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
# WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
# ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
# OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
#
#-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
#
#

# Source the sysexits library:
. ${LOCALBASE}/lib/sysexits.sh


this=${0##*/}
version='${VERSION}'
trap_addresses='${TRAP_ADDRESSES}'
spamd_db='${DAEMON_DB}'
tmp_dir=${TMPDIR:-/tmp}

readonly this version trap_addresses spamd_db tmp_dir


function usage
{
	[[ -n ${1} && ${1} -eq 0 ]] ||
	{
		_exit_code=${EX_USAGE}
		_print_args='-u2'
	}

	_usage="$0: ${this} [-hv] (add|delete|dump|load) [-l]"
	_usage="${_usage} (-stw) [-p pattern] [-f file]\n"
	_usage="${_usage}\t-h (help) -v (version) -l (log)"
	_usage="${_usage} -s (SPAMTRAP) -t (TRAPPED) -w (WHITE)"
	print ${_print_args} -- ${_usage}

	exit ${_exit_code}
}


# Set the operation mode:
case ${1} in
	add|delete|dump|load)
		mode=${1}
		spamdb_args="-$(print ${mode} | cut -c 1)"
		shift
		;;
esac


while getopts "f:hlp:stvw" opt
do
	case ${opt} in
		f)
			file="${OPTARG}"
			;;
		h)
			usage ${EX_OK}
			;;
		l)
			log='aye'
			;;
		p)
			pattern="${OPTARG}"
			;;
		s)
			db_type='SPAMTRAP'
			spamdb_args="${spamdb_args} -T"
			;;
		t)
			db_type='TRAPPED'
			spamdb_args="${spamdb_args} -t"
			;;
		v)
			print "Version: ${version}"
			exit
			;;
		w)
			db_type='WHITE'
			;;
		\?)
			usage
			;;
	esac
done


# Check valid mode arg & spamd_db permissions
case ${mode} in
	add|delete|load)
		[[ -w ${spamd_db} ]] ||
		{
			print -u2 "${spamd_db} not writable by ${LOGNAME}"
			print -u2 "See the SECURITY section of ${this}(${MAN_SECTION})"
			exit ${EX_NOPERM}
		}
		;;
	dump)
		[[ -r ${spamd_db} ]] ||
		{
			print -u2 "${spamd_db} not readable by ${LOGNAME}"
			print -u2 "See the SECURITY section of ${this}(${MAN_SECTION})"
			exit ${EX_NOPERM}
		}
		;;
	*)
		usage
		;;
esac


# Check db_type arg
case ${mode} in
	add|delete)
		[[ -n ${db_type} ]] || usage
		;;
esac


# Check file arg
case ${mode} in
	add)
		case ${db_type} in
			SPAMTRAP)
				[[ -n ${file} ]] || file=${trap_addresses}
				;;
			TRAPPED|WHITE)
				[[ -n ${file} ]] ||
				{
					print -u2 "\n*** IP address file needed. ***\n"
					usage
				}
				;;
		esac
		;;

	load)
		[[ -n ${file} ]] ||
		{
			print -u2 "\n*** Dump file needed. ***\n"
			usage
		}
		;;
esac


# If there is a file specified
[[ -n ${file} ]] &&
{
	# Check it exists & is readable
	[[ -f ${file} && -r ${file} ]] ||
	{
		print -u2 "${file} isn't a (readable) file."
		exit ${EX_NOINPUT}
	}
}


function do_add_or_delete
{
	spinner[0]='/'
	spinner[1]='-'
	spinner[2]='\\'
	spinner[3]='|'
	spindex=0

	print -n 'Parsing:  '

	while read key
	do
		printf "\b"${spinner[${spindex}]}
		(( spindex=(spindex+1)%${#spinner[*]} ))

		${DAEMON_DB_TOOL} ${spamdb_args} ${key} &&
		{
			let processed++
			[[ -f ${log} ]] && print ${key} >> ${log}
		}

	done < ${keys}

	print "\b\b\b\b\b\b\b\b\b\b\b\b"
	print "Processed: ${processed}"
}


function get_keys
{
	keys=$(mktemp)
	${DAEMON_DB_TOOL} | grep ^${db_type} | cut -d '|' -f2 > ${keys}
	[[ ${mode} == 'delete' ]] &&
	{
		[[ -s ${keys} ]] || return
	}

	[[ -n ${file} ]] &&
	{
		file_keys=$(mktemp)
		egrep -v '^#|^$' ${file} | tr -d ' ' | sort -fu -o ${file_keys}
		[[ -s ${file_keys} ]] || return

		sort_keys=$(mktemp)
		sort -fu -o ${sort_keys} ${keys}

		_comm_args='-f -2'
		case ${mode} in
			add)
				_comm_args="${_comm_args} -3"
				;;
			delete)
				_comm_args="${_comm_args} -1"
				;;
		esac
		comm ${_comm_args} ${file_keys} ${sort_keys} | tr -d '\t' > ${keys}
	}

	[[ -n ${pattern} && -s ${keys} ]] &&
	{
		pattern_keys=$(mktemp)
		grep -i -- ${pattern} ${keys} > ${pattern_keys}
		mv ${pattern_keys} ${keys}
	}

	[[ -s ${keys} ]]
}


do_load()
{
	load_spam_traps=$(mktemp)
	awk -F \| '/^SPAMTRAP/ { print $2 }' ${file} > ${load_spam_traps}
	$0 add -l -s -f ${load_spam_traps}
	
	load_trapped=$(mktemp)
	awk -F \| '/^TRAPPED/ { print $2 }' ${file} > ${load_trapped}
	$0 add -l -t -f ${load_trapped}
	
	load_whites=$(mktemp)
	awk -F \| '/^WHITE/ { print $2 }' ${file} > ${load_whites}
	$0 add -l -w -f ${load_whites}
}


function die
{
	_exit_code=${1:-0}
	[[ $# -eq 0 ]] ||
	{
		shift
		print -u2 -- $*
	}

	[[ -f ${log} && ${_exit_code} -eq 0 ]] &&
	{
		mv ${log} ${tmp_dir} && print "Log: ${tmp_dir}/${log##*/}"
	}
	[[ -f ${dump} ]] &&
	{
		mv ${dump} ${tmp_dir} && print "Dump: ${tmp_dir}/${dump##*/}"
	}

	[[ -d ${TMPDIR} ]] && rm -r ${TMPDIR}
	exit ${_exit_code}
}


TMPDIR=$(mktemp -d) ||
{
	unset TMPDIR
	die ${EX_CANTCREAT} 'Failed to create tmp dir'
}
export TMPDIR


case ${mode} in
	add|delete)
		[[ -n ${log} ]] && log=$(mktemp -t ${this}_${mode}_${db_type}.log.XXXXXXX)
		get_keys || die ${EX_TEMPFAIL} 'No keys found'
		do_add_or_delete
		;;

	dump)
		dump=$(mktemp -t ${this}.${mode}.XXXXXXX)
		${DAEMON_DB_TOOL} | sort -f -o ${dump}
		;;

	load)
		do_load
		;;

esac


die

